#/******************************************************************************
#*                              __subsf3
#*******************************************************************************
#* Subtract two floating point values: C = A - B
#*
#* Inputs:  [R1,R2] - 32-bit float, value A
#*          [R3,R4] - 32-bit float, value B
#*
#* Returns: [R1,R2] - 32-bit result, value C
#******************************************************************************/
  def __subsf3
  even
__subsf3:
  ai r3, 0x8000  # Negate B
  # Fall though and compute A+(-B)


#/******************************************************************************
#*                              __addsf3
#*******************************************************************************
#* Add two floating point values: C = A + B
#*
#* Inputs:  [R1,R2] - 32-bit float, value A
#*          [R3,R4] - 32-bit float, value B
#*
#* Returns: [R1,R2] - 32-bit result, value C
#******************************************************************************/
  def __addsf3
  even
__addsf3:
  # Store return pointer for later
  dect r10
  mov r11, *r10

  # If one of the arguments is NaN, return NaN
  bl @__using_nan
  jeq addsub_return_nan

  # Convert to cannonical form
  mov r1, r1
  jlt add_nega
  mov r3, r3
  jlt add_posa_negb
  # A>0, B>0 : C = (A)+(B)
  bl @add_values
  jmp addsub_return_a

add_posa_negb:
  # A>0, B<0 : C = (A)+(-B) = (A)-(B')
  ai r3, 0x8000   # B' = -B
  bl @sub_values
  jmp addsub_return_a

add_nega:
  mov r3, r3
  jlt add_nega_negb
  # A<0, B>0 : C = (-A)+(B) = (B)-(A)
  ai r1, 0x8000   # A = -A
  bl @addsub_swap_args
  bl @sub_values
  jmp addsub_return_a
  
add_nega_negb:
  # A<0, B<0 : C = (-A)+(-B) = -((A')+(B'))
  ai r1, 0x8000   # A' = -A
  ai r3, 0x8000   # B' = -B
  bl @add_values
  ai r1, 0x8000   # C' = -C
  jmp addsub_return_a

###############################################################################
add_values:
  dect r10
  mov r11, *r10

  # Signs are positive, put larger value in B slot
  bl @__ucomparesf
  jlt add_no_swap
  bl @addsub_swap_args
  
add_no_swap:  
  bl @__remove_exponents
  ci r3, 0x7F80
  jeq addsub_return_inf   # a+inf=inf
 
  # Larger value now in B slot, shift A to match exponent of B
  mov r6, r0
  s   r5, r0            # Shift = exp(B) - exp(A)
  srl r0, 7             # Move result into least significant bits of R0
  bl  @__shiftr_32      # A >>= exp(B) - exp(A)

  # Add A and B
  a r3, r1
  a r4, r2
  jnc add_no_inc
  inc r1
add_no_inc:

  # Correct exponent
  mov r6, r3
  mov *r10+, r11
  b @__normalize


###############################################################################
sub_values:
  dect r10
  mov r11, *r10

  # Signs are positive, put larger value in A slot
  bl @__ucomparesf
  jgt sub_no_swap
  # Swap arguments, negate result
  # C=-(B-A)
  bl @addsub_swap_args
  bl @sub_values
  ai r1, 0x8000
  jmp addsub_return_a

sub_no_swap:
  bl @__remove_exponents  # Exponents put into r5 and r6
  li r7, 0x7F80           # Exponent for infinity
  c  r7, r6
  jeq addsub_return_nan  # inf-inf=nan
  c  r7, r5
  jeq addsub_return_inf  # inf-b=inf

  # Put larger value in B slot, shift A to match exponent of B
  bl @addsub_swap_args
  mov r6, r0
  s r5, r0             # Shift = exp(B) - exp(A)
  srl r0, 7            # Move result into least significant bits of R0
  bl @__shiftr_32      # A >>= exp(B) - exp(A)

  # Subtract A from B
  s r1, r3
  s r2, r4
  jnc sub_no_dec
  dec r1
sub_no_dec:
  mov r3, r1
  mov r4, r2

  # Correct exponent
  mov r6, r3
  mov *r10+, r11
  b @__normalize

###############################################################################
# Common routines for addition and subtraction

addsub_return_inf:
  li r1, 0x7F80
  clr r2
  jmp addsub_return_a

addsub_return_nan:
  li r1, 0x7FC0
  clr r2
  jmp addsub_return_a

addsub_return_b:
  mov r3, r1
  mov r4, r2
  
addsub_return_a:
  mov *r10+, r11
  b *r11

addsub_swap_args:
  mov r1, r0  #// Swap high words
  mov r3, r1
  mov r0, r3
  mov r2, r0  #// Swap low words
  mov r4, r2
  mov r0, r4
  b *r11

