#/******************************************************************************
#*                              __mult32
#*******************************************************************************
#* Do 32-bit by 32-bit multiply, but only retain the most significant 25 bits
#*
#* Inputs:  [R1,R2] - 32-bit, value A
#*          [R3,R4] - 32-bit, value B
#*
#* Returns: [R1,R2] - 32-bit result, value C
#******************************************************************************/
  def __mult32
  even
__mult32:
# Do 32-bit multiply
# This is a 64-bit result, but many of the bytes can be ignored
# espicially since we only need the most significant 25 bits
# 
#     .------.--- We want this part
#     |      | 
# 7 6 5 4 3 2 1 0    bytes   A = Ahi(9bits), Alo(16Bits)
# ____xxxx          Ahi*Bhi (18 bits)
#     __xxxxxx      Ahi*Blo (25 bits)   "_" Known to be zero value
#     __xxxxxx      Alo*Bhi (25 bits)   "x" Unknown value
#         xxxxxxxx  Alo*Blo (32 bits)
#
# Register allocation:
#   output  -> [r1,r2]
#   A value -> [r1,r2]  
#   B value -> [r3,r4] = [Bhi,Blo]
#   A copy  -> [r5,r6] = [Ahi,Alo]
#   Temp    -> [r7,r8]

  mov r1, r5  # \
  mov r2, r6  # / Move A to [r5,r6]
  mov r6, r7  # \
  mpy r4, r7  # / Temp = Alo*Blo
  clr r1      # \
  mov r7, r2  # / Out = (Alo*Blo) >> 16
  mov r6, r7  # \
  mpy r3, r7  # / Temp = Alo*Bhi
  mov r7, r1  # \
  a   r8, r2  # | Out += Alo*Bhi
  jnc $+4     # |
  inc r1      # /
  mov r5, r7  # \
  mpy r4, r7  # / Temp = Ahi*Blo
  a   r7, r1  # \
  a   r8, r2  # | Out += Ahi*Blo
  jnc $+4     # |
  inc r1      # /
  mpy r3, r5  # A = Ahi*Bhi
  a   r6, r1  # Out += (Ahi*Bhi) << 16
  b *r11

