#/******************************************************************************
#*                              __powisf2
#*******************************************************************************
#* Raise floating point number to integer exponent: C = A ^ B
#* NOTE: Denormals are not supported by this function
#*
#* Inputs:  [R1,R2] - 32-bit float, value A

#*          [R3,R4] - 32-bit int, value B
#*
#* Returns: [R1,R2] - 32-bit float result, value C
#******************************************************************************/
  def __powisf2
  even
__powisf2:
  # Store return pointer for later
  ai r10, -14
  mov r10, r0
  mov r11, *r0+   # +0
  mov r9,  *r0+   # +2
  mov r13, *r0+   # +4
  mov r14, *r0+   # +6
  mov r15, *r0+   # +8
  # Chigh  +10
  # Clow   +12

  # Store A in [r14, r15]
  mov r1, r14
  mov r2, r15

  # Store C=(float)1 in [10(r10), 12(r10)]
  li  r0, 0x3f80
  mov r0, @10(r10)
  clr @12(r10)

  # Store B in [r9, r13]
  mov r4, r13
  mov r3, r9

  # If B is negative, negate B and invert A
  jlt pow_neg_exp
  jmp pow_top

pow_neg_exp:
  # Negate B (B=-B)
  inv r9
  neg r13
  jnc $+4
  inc r9

  # Invert A (A=1/A)
  mov r0, r1   # \ (float)1 as arg 1
  clr r2       # /
  mov r14, r3  # \ A as arg 2
  mov r15, r4  # /
  bl @__divsf3
  mov r1, r14  # \ Store result in A
  mov r2, r15  # /
  
  #########################################3
  # Pseudocode:
  #   pow_top:
  #     B/=2
  #     if(B&1)
  #       C*=A
  #     if(B==0)
  #       goto pow_done
  #     A *= A
  #     goto pow_top
  #   pow_done:

pow_top:
  #   B /= 2;
  srl r13, 1   # Just low word for now, we'll shift high word later
 
  #   if (B & 1)
  jnc pow_skip_cxa

  #     C *= A;
  mov r14, r1       # \ A as arg 1
  mov r15, r2       # /
  mov @10(r10), r3  # \ C as arg 2
  mov @12(r10), r4  # /
  bl @__mulsf3
  mov r1, @10(r10)  # \ Store result in C
  mov r2, @12(r10)  # /
pow_skip_cxa:

  # Shift high word of B
  srl r9, 1
  jnc pow_no_ch_bit
  ori r13, 0x8000
pow_no_ch_bit: 

  # if(B == 0)
  #   goto pow_done
  mov r9, r1
  soc r13, r1
  jeq pow_done

  # A *= A;
  mov r14, r1  # \ A as arg 1
  mov r15, r2  # /
  mov r14, r3  # \ A as arg 2
  mov r15, r4  # /
  bl @__mulsf3
  mov r1, r14  # \ Store result in A
  mov r2, r15  # /

  # Goto top of loop
  jmp pow_top

pow_done:
  # Copy C to return position
  mov @10(r10), r1
  mov @12(r10), r2

pow_end:
  # Clean up stack and return
  mov *r10+, r11
  mov *r10+, r9
  mov *r10+, r13
  mov *r10+, r14
  mov *r10, r15
  ai r10, 6
  b *r11

