#/******************************************************************************
#*                            __remove_exponents
#*******************************************************************************
#* Convert A and B to remove exponents and add implied bits
#* Put exponents in R5 and R6
#*
#* Inputs:  [R1,R2] - 32-bit float
#*          [R3,R4] - 32-bit float
#*
#* Returns: [R1,R2] - 32-bit stripped float
#*          [R3,R4] - 32-bit stripped float
#*           R5     - Stripped exponent
#*           R6     - Stripped exponent
#*
#* Clobbers: R7
#******************************************************************************/
  def __remove_exponents
  even
__remove_exponents:
  li  r7, 0x807F       # Mask for exponent bits, 0x807F = ~0x7F80
  mov r1, r5
  sb  r1, r1           # Clear sign and exponent bits for A
  szc r7, r5           # Isolate exponent for A in R5
  jeq re_do_b          # If zero exponent, don't set bit
  ori r1, 0x0080       # Set implied bit in A
re_do_b:
  mov r3, r6
  sb  r3, r3           # Clear sign and exponent bits for B
  szc r7, r6           # Isolate exponent for B in R6
  jeq re_exit          # If zero exponent, don't set bit
  ori r3, 0x0080       # Set implied bit in B
re_exit:
  b *r11

